-- Test contact_tag
set client_min_messages to warning;
create extension if not exists pgtap;
reset client_min_messages;

begin;

select plan(23);

set search_path to numerus, auth, public;

select has_table('contact_tag');
select has_pk('contact_tag' );
select col_is_pk('contact_tag', array['contact_id', 'tag_id']);
select table_privs_are('contact_tag', 'guest', array []::text[]);
select table_privs_are('contact_tag', 'invoicer', array ['SELECT', 'INSERT', 'UPDATE', 'DELETE']);
select table_privs_are('contact_tag', 'admin', array ['SELECT', 'INSERT', 'UPDATE', 'DELETE']);
select table_privs_are('contact_tag', 'authenticator', array []::text[]);

select has_column('contact_tag', 'contact_id');
select col_is_fk('contact_tag', 'contact_id');
select fk_ok('contact_tag', 'contact_id', 'contact', 'contact_id');
select col_type_is('contact_tag', 'contact_id', 'integer');
select col_not_null('contact_tag', 'contact_id');
select col_hasnt_default('contact_tag', 'contact_id');

select has_column('contact_tag', 'tag_id');
select col_is_fk('contact_tag', 'tag_id');
select fk_ok('contact_tag', 'tag_id', 'tag', 'tag_id');
select col_type_is('contact_tag', 'tag_id', 'integer');
select col_not_null('contact_tag', 'tag_id');
select col_hasnt_default('contact_tag', 'tag_id');


set client_min_messages to warning;
truncate contact_tag cascade;
truncate contact cascade;
truncate tag cascade;
truncate contact cascade;
truncate company_user cascade;
truncate company cascade;
truncate payment_method cascade;
truncate auth."user" cascade;
reset client_min_messages;

insert into auth."user" (user_id, email, name, password, role, cookie, cookie_expires_at)
values (1, 'demo@tandem.blog', 'Demo', 'test', 'invoicer', '44facbb30d8a419dfd4bfbc44a4b5539d4970148dfc84bed0e', current_timestamp + interval '1 month')
     , (5, 'admin@tandem.blog', 'Demo', 'test', 'admin', '12af4c88b528c2ad4222e3740496ecbc58e76e26f087657524', current_timestamp + interval '1 month')
;

set constraints "company_default_payment_method_id_fkey" deferred;

insert into company (company_id, business_name, vatin, trade_name, phone, email, web, address, city, province, postal_code, country_code, currency_code, default_payment_method_id)
values (2, 'Company 2', 'XX123', '', '555-555-555', 'a@a', '', '', '', '', '', 'ES', 'EUR', 222)
     , (4, 'Company 4', 'XX234', '', '666-666-666', 'b@b', '', '', '', '', '', 'FR', 'USD', 444)
;

insert into payment_method (payment_method_id, company_id, name, instructions)
values (444, 4, 'cash', 'cash')
     , (222, 2, 'cash', 'cash')
;

set constraints "company_default_payment_method_id_fkey" immediate;

insert into company_user (company_id, user_id)
values (2, 1)
     , (4, 5)
;

insert into contact (contact_id, company_id, business_name, vatin, trade_name, phone, email, web, address, city, province, postal_code, country_code)
values (6, 2, 'Contact 1', 'XX555', '', '777-777-777', 'c@c', '', '', '', '', '', 'ES')
     , (8, 4, 'Contact 2', 'XX666', '', '888-888-888', 'd@d', '', '', '', '', '', 'ES')
;

insert into tag (tag_id, company_id, name)
values (14, 2, 'web')
     , (15, 2, 'design')
     , (16, 4, 'product')
     , (17, 4, 'development')
     , (18, 4, 'something-else')
     , (19, 4, 'design')
;

insert into contact_tag (contact_id, tag_id)
values (6, 14)
     , (6, 15)
     , (8, 18)
;

prepare contact_tag_data as
select contact_id, tag_id
from contact_tag
;

set role invoicer;
select is_empty('contact_tag_data', 'Should show no data when cookie is not set yet');
reset role;

select set_cookie('44facbb30d8a419dfd4bfbc44a4b5539d4970148dfc84bed0e/demo@tandem.blog');
select bag_eq(
	'contact_tag_data',
	$$ values ( 6, 14 )
	        , ( 6, 15 )
	$$,
	'Should only list contact tags of the companies where demo@tandem.blog is user of'
);
reset role;

select set_cookie('12af4c88b528c2ad4222e3740496ecbc58e76e26f087657524/admin@tandem.blog');
select bag_eq(
	'contact_tag_data',
	$$ values ( 8, 18 )
	$$,
	'Should only list contact tags of the companies where admin@tandem.blog is user of'
);
reset role;

select set_cookie('not-a-cookie');
select throws_ok(
	'contact_tag_data',
	'42501', 'permission denied for table contact_tag',
	'Should not allow select to guest users'
);


select *
from finish();

rollback;

